/*
autor:		thomas rausch
		0726439 | 543

file:		create.sql
created: 	2009-11-06
*/
-- Adresse (aid, plz, ort, strasse, nummer)
CREATE TABLE Adresse (
	aid		SERIAL      PRIMARY KEY,
	plz		SMALLINT    NOT NULL,
	ort		VARCHAR(40) NOT NULL,
	strasse		VARCHAR(70) NOT NULL,
	nummer		VARCHAR(20) NOT NULL,		
	UNIQUE (plz,ort,strasse,nummer)
);

-- Standort(sid, bezeichnung, adresse:Adresse.aid, leiter:Angestellter.svnr)
CREATE TABLE Standort (
	sid 		SERIAL      PRIMARY KEY,
	bezeichnung 	TEXT        NOT NULL,
	adresse 	INTEGER     REFERENCES Adresse(aid),
	leiter		NUMERIC(12) --REFERENCES Angestellter(svnr)
);

-- Angestellter(svnr, nname, vname, gebdat, gehalt, adresse:Adresse.aid, arbeitet:Standort.sid)
CREATE TABLE Angestellter (
	svnr		NUMERIC(12)  PRIMARY KEY,
        nname		VARCHAR(20)  NOT NULL,
        vname		VARCHAR(20)  NOT NULL,
	gebdat		DATE         NOT NULL,
	gehalt		NUMERIC(7,2) NOT NULL,
	adresse		INTEGER      REFERENCES Adresse(aid),
	arbeitet	INTEGER      REFERENCES Standort(sid) DEFERRABLE INITIALLY DEFERRED
);

ALTER TABLE Standort ADD CONSTRAINT standort_leiter
	FOREIGN KEY (leiter) REFERENCES Angestellter(svnr)
	DEFERRABLE INITIALLY DEFERRED;

-- Manager(svnr:Angestellter.svnr, bonus)
CREATE TABLE Manager (
	svnr		NUMERIC(12)  PRIMARY KEY REFERENCES Angestellter,
	bonus		NUMERIC(7,2) NOT NULL
);

-- Kunde(knr, nname, vname, adresse:Adresse.aid)
CREATE TABLE Kunde (
	knr		NUMERIC(7)  PRIMARY KEY,
	nname		VARCHAR(20) NOT NULL,
	vname		VARCHAR(20) NOT NULL,
	adresse		INTEGER     REFERENCES Adresse(aid)
);

-- Artikel(ean, bezeichnung, typ
CREATE TABLE Artikel (
	ean             NUMERIC(13) PRIMARY KEY,
	bezeichnung	TEXT        NOT NULL,
	typ		TEXT        NOT NULL
);

-- serie(vorgaenger:Artikel.ean, nachfolger:Artikel.ean)
CREATE TABLE serie (
	vorgaenger	NUMERIC(13) REFERENCES Artikel(ean),
	nachfolger	NUMERIC(13) REFERENCES Artikel(ean),
	PRIMARY KEY (vorgaenger, nachfolger)
);

-- Land(code, name, kontinent)
CREATE TABLE Land (
	code		CHAR(3)     PRIMARY KEY, -- ISO 3166-1
	name		VARCHAR(50) NOT NULL,
	kontinent	VARCHAR(15) NOT NULL,
	CHECK (kontinent IN ('Afrika', 'Asien', 'Australien', 'Europa', 'Nordamerika', 'Südamerika'))
);

-- Preis(datum, artikel:Artikel.ean, land:Land.code, wert, steuer)
CREATE TABLE Preis (
	datum		DATE,
	artikel		NUMERIC(13)  REFERENCES Artikel(ean),
	land		CHAR(3)      REFERENCES Land(code),
	wert		NUMERIC(7,2) NOT NULL,
        steuer          NUMERIC(3)   NOT NULL, -- prozent
	CHECK (steuer BETWEEN 0 AND 100),
	PRIMARY KEY (datum, artikel, land)
);

-- Bestellung(bid, zeitpunkt, bezahlt, datum, knr:Kunde.knr)
CREATE TABLE Bestellung (
	bid		NUMERIC(10) PRIMARY KEY,
	zeitpunkt	TIMESTAMP   NOT NULL,
	bezahlt		VARCHAR(4)  NOT NULL,
	datum		DATE        NOT NULL,
	knr		NUMERIC(7)  REFERENCES Kunde,
	CHECK (bezahlt IN ('JA', 'NEIN'))
);

-- umfasst(bid:Bestellung.bid, artikel:Preis.artikel, land:Preis.land, datum:Preis.datum, anzahl)
CREATE TABLE umfasst (
	bid		NUMERIC(10) REFERENCES Bestellung,
	datum		DATE,
	artikel		NUMERIC(13),
	land		CHAR(3),
	anzahl		SMALLINT    NOT NULL,
	PRIMARY KEY (bid, datum, artikel, land),
	FOREIGN KEY (datum, artikel, land) REFERENCES Preis
);

-- sequences
CREATE SEQUENCE kunde_knr_seq
	START 1000001
	INCREMENT 10
	MINVALUE 1000001
	NO MAXVALUE;

CREATE SEQUENCE bestellung_bid_seq
	START 1000000001
	INCREMENT 10
	MINVALUE 1000000001
	NO MAXVALUE;



